function [mesh]=readMesh(fname)
% [MESH]=READMESH(FNAME) Parses mesh format (.dat) including a skeleton
%
% INPUT:
%   fname: filename to the mesh
% OUTPUT:
%   mesh: struct containing the following fields:
%         vertices : 3 x N double array, vertex positions of N vertices
%         faces    : 3 x F double array, indices into vertices to define faces
%         skinningJoints : W x N double array containing the joint IDs to
%            which each vertex is assigned to.
%         skinningWeights : W x N  double array containing skinning
%            weights. Hereby, a weighted asociation of the vertices to the
%            joints is defined.
%         skel : skeleton struct of the mesh as a struct contatining the fields
%                nodes: Array of K structs containing the fields
%                       ID: Node ID (ID 1 == root)
%                       parent: parent ID
%                       children: ids of childs
%                       position: 3x1 vector containing 3d position of the joint
%                       direction: 3x1 rotation axis definition
%                       moment: 3x1 cross(position, direction)
%                influencedBy: KxK logical matrix, (i,j) == true j is on the
%                              way from node i to the root.
%         boundingBox: 6x1 boundingBox of the mesh, xMin xMax yMin yMax zMin zMax
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
% Written by: Andreas Baak
% Revision:   1.0
% Date:       15.07.2009
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this
% software and its documentation for any non-commercial purpose is hereby
% granted without fee, provided that the above copyright notice appear in
% all copies and that both that copyright notice and this permission
% notice appear in supporting documentation, and that the name of the
% author not be used in advertising or publicity pertaining to distribution
% of the software without specific, written prior permission.
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
isNoBallModel=false;
fid=fopen(fname,'r');
if (fid==-1)
    error(['Could not open the file ' sensorFile])
    return;
end
l = fgetl(fid);
[dim, count]= sscanf(l, '%d', inf);
% distinguish models that have skinning weights and models that don't.
if count < 4
    numSkinningWeights= 0;
else
    numSkinningWeights= dim(4);
end
if numSkinningWeights == 0
    vertexData = fscanf(fid,'%f',[4,dim(1)]);
else
    vertexData = fscanf(fid,'%f',[3 + 2*numSkinningWeights,dim(1)]);
end
mesh.vertices = vertexData(1:3, :)/1000; % convert from millimeters to meters
mesh.faces=fscanf(fid,'%f',[3, dim(2)]) + 1; % correct for matlab indexing
if numSkinningWeights == 0
    mesh.skinningJoints  = vertexData(4, :)+1; % correct for matlab indexing
    mesh.skinningWeights = ones(1, dim(1));
else
    mesh.skinningJoints = vertexData(4:2:end, :) + 1; %correct for matlab indexing
    mesh.skinningWeights = vertexData(5:2:end, :);
end
%warning('readMesh:ChangeCoorinates', 'Temporary fix: Changing x-coordinates.');
%mesh.vertices(1,:) = mesh.vertices(1,:)*(-1);
% we want to create a root node in the origin of the area that is
% controlled by the root node. To this end, compute the center of all
% vertices that are assigned to the root node and take the center of the
% bounding box thereof as the new origin for the model.
rootNode = min(min(mesh.skinningJoints));
bb = computeBoundingBoxVertices(mesh.vertices(:, any(mesh.skinningJoints(1,:)==rootNode, 1))); %%bounding box of only vertices directly influenced by root joint
% root = mean(mesh.vertices, 1);
root = [(bb(1) + bb(2))/2
    (bb(3) + bb(4))/2
    (bb(5) + bb(6))/2];
mesh.center=root;
mesh.vertices = mesh.vertices - repmat(root, 1, dim(1) );
emptyNode = struct('ID', [], 'parent', [], 'children', [], 'position', [], 'direction', [], 'moment', [], 'isBallJoint', []);
skel = struct('nodes', emptyNode);
% create root node
skel.nodes(1).ID = 1; % the node with ID 1 is the root.
skel.nodes(1).parent = 0;
skel.nodes(1).position = [0 0 0]';
skel.nodes(1).direction = [0 0 0]';
skel.nodes(1).isBallJoint=0;
skel.nodes(2:dim(3)+1)=emptyNode;
% now parse the skeleton nodes
if (isNoBallModel)
    skelData = fscanf(fid, '%d %f %f %f %f %f %f %d', 8*dim(3));
    skelData = reshape(skelData , 8, dim(3))';
    % Add the ball joint flag = 0
    skelData(:,9)=0;
else
    skelData = fscanf(fid, '%d %f %f %f %f %f %f %d %d', 9*dim(3));
    skelData = reshape(skelData , 9, dim(3))';
end
for i=1:dim(3)
    ID = skelData(i, 1) + 1; % correct for matlab indexing
    direction = skelData(i, 2:4)';
    position = (skelData(i, 5:7)/1000 - root')'; % convert form millimeters to meters.
    parent = skelData(i, 8) + 1; % correct for matlab indexing
    moment = cross(position, direction);
    isBallJoint=skelData(i,9);     %Uncomment for balljoints
    skel.nodes(parent).children = [skel.nodes(parent).children ID];
    if (ID ~= (numel(skel.nodes)+1))
        % error('The joint IDs in the skeleton definintion have to be increasing integers, beginning from 1, without any gaps.');
    end
    skel.nodes(ID).ID=ID;
    skel.nodes(ID).direction=direction;
    skel.nodes(ID).position=position;
    skel.nodes(ID).parent=parent;
    skel.nodes(ID).moment=moment;
    skel.nodes(ID).isBallJoint=isBallJoint;
end
numNodes = numel(skel.nodes);
nodeParent= [skel.nodes.parent];
influencedBy = false(numNodes, numNodes);
% this matrix is true in (i,j), if node i is influenced by node j. That is,
% if you start your way from node i to the root, then node j is on that
% way.
for i=1:numNodes
    for j=1:numNodes
        if i == j
            influencedBy(i,j) = true;
        else
            if isParentOf(j, i)
                influencedBy(i, j) = true;
            end
        end
    end
end

mesh.skel = skel;
mesh.skel.influencedBy  = influencedBy;
mesh.boundingBox = computeBoundingBoxVertices(mesh.vertices);
fclose(fid);
    function res = isParentOf(parent, joint)
        if (joint == 1)
            res = false; % the root does not have a parent.
        elseif (nodeParent(joint) == parent)
            res = true;
        else
            res = isParentOf(parent,nodeParent(joint));
        end
    end
end